package com.tzld.longarticle.recommend.server.service.filter;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.service.ServiceBeanFactory;
import com.tzld.longarticle.recommend.server.service.filter.strategy.CategoryStrategy;
import com.tzld.longarticle.recommend.server.service.filter.strategy.DuplicateStrategy;
import com.tzld.longarticle.recommend.server.service.filter.strategy.HistoryTitleStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class FilterService {
    private final ExecutorService pool = ThreadPoolFactory.filterPool();

    public FilterResult filter(FilterParam param) {
        List<String> contentIds = contentFilter(param);

        return new FilterResult(contentIds);
    }

    private List<String> contentFilter(FilterParam param) {

        List<FilterStrategy> strategies = getStrategies(param);
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        List<Future<List<String>>> futures = new ArrayList<>();
        for (final FilterStrategy strategy : strategies) {
            Future<List<String>> future = pool.submit(() -> {
                List<String> result = strategy.filter(param);
                cdl.countDown();
                return result;
            });
            futures.add(future);
        }
        try {
            cdl.await(1000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("filter error", e);
            return null;
        }

        List<List<String>> contentIds = new ArrayList<>();
        for (Future<List<String>> f : futures) {
            try {
                contentIds.add(f.get());
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }
        if (CollectionUtils.isEmpty(contentIds)) {
            return Collections.emptyList();
        }
//        List<String> result = Lists.newArrayList(param.getContents());
//        for (int i = 0; i < contentIds.size(); ++i) {
//            result.retainAll(contentIds.get(i));
//        }
        return null;
    }

    private List<FilterStrategy> getStrategies(FilterParam param) {
        List<FilterStrategy> strategies = new ArrayList<>();
        strategies.add(ServiceBeanFactory.getBean(HistoryTitleStrategy.class));
        strategies.add(ServiceBeanFactory.getBean(DuplicateStrategy.class));
        strategies.add(ServiceBeanFactory.getBean(CategoryStrategy.class));

        return strategies;
    }
}
